#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
from struct import *

def glmload(filename, raw_filename):
    #:param str filename: ファイルパス
    #:param str raw_filename: 選択ファイル名
    #:rtype: gimp.Image
    #:return: 読み込んだ画像
    fp = open(filename, 'rb')

    header = fp.read(8)
    if not header[0:4] == "GR65":
        gimp.message("GLMファイルではありません")
        fp.close()
        return

    x0 = unpack('>H', fp.read(2))[0]
    y0 = unpack('>H', fp.read(2))[0]
    x1 = unpack('>H', fp.read(2))[0]
    y1 = unpack('>H', fp.read(2))[0]

    width = x1-x0
    height = y1-y0
    
    #パレット計算用テーブル 5bit+1bit を 8bit に
    octa = [0]*64
    for i in range(64):
        octa[i] = int(round(i * 255 * 1.0 / 63))
        
    #GIMPに画像を作成
    image = gimp.Image(width, height, RGB)
    image.filename = filename

    #レイヤー作成
    layer = pdb.gimp_layer_new(image, width, height, RGB, 'GLX', 100, NORMAL_MODE )

    image.add_layer(layer, 0)

    #画像展開
    for y in range(height):
        for x in range(width):
            p =  unpack('>H', fp.read(2))[0]
            g = ( (p & 0b1111100000000000)>>10 ) + (p & 0b0000000000000001)
            r = ( (p & 0b0000011111000000)>>5 ) + (p & 0b0000000000000001)
            b = p & 0b0000000000111111
            color = [octa[r],octa[g],octa[b]]
            layer.set_pixel( x,y, color )
            x +=1
        y +=1
	gimp.progress_update(float(y)/float(height)) # ループに対応したプログレスバーの更新

        
    return image

# 登録ハンドラ
def register_load_handlers():
    gimp.register_load_handler('python_fu_GLM_load', 'glm', '')
    pdb['gimp-register-file-handler-mime']('python_fu_GLM_load', 'image/x68glm')
#    pdb['gimp-register-thumbnail-loader']('python_fu_GLM_load', '')


register(
    "python_fu_GLM_load", #コマンド名
    "Load X68 GLM image", #プロシージャブラウザに表示されるプラグインに関する情報
    "X68用GLM形式画像を読み込みます", #プラグインのヘルプ
    "Akira M", #プラグインの作成者
    "Akira M", #プラグインの著作権保有者
    "2017", #年
    "X68 GLM", #メニューの中でプラグインに使用されるラベル
    None, #プラグインで処理する対象となる画像のタイプ
    [   #input args. Format (type, name, description, default [, extra])
        (PF_STRING, 'filename', 'The name of the file to load', None),
        (PF_STRING, 'raw-filename', 'The name entered', None),
    ], #プラグインのメソッドのパラメーター
    [(PF_IMAGE, 'image', 'Output image')], #プラグインのメソッドの結果 Format (type, name, description) 
    glmload,
    on_query = register_load_handlers,
    menu = "<Load>/Load x68GLM..",
)

main()
